import urllib.parse
from dataclasses import asdict
from typing import Any

import requests

from druidapi import configuration


class CommonClient:
    session: requests.Session
    base_url: str

    """Low-level REST client to the Druid API. Does only provide a wrapper for the requests package with predefined
    URLs and arguments."""
    def __init__(self, config: configuration.DruidApiClientConfiguration):
        self.base_url = config.endpoint
        self.session = requests.Session()

        if config.username is not None and config.password is not None:
            self.session.auth = (config.username, config.password)
        if config.proxies is not None:
            self.session.proxies = asdict(config.proxies)
        self.session.verify = config.verify_ssl

        if not self.session.verify:
            import urllib3  # type: ignore
            urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

    def _build_url(self, url: str) -> str:
        return urllib.parse.urljoin(self.base_url, url)

    def _request(self, method: str, url: str, *kargs: Any, **kwargs: Any) -> requests.Response:
        return self.session.request(method, self._build_url(url), *kargs, **kwargs)

    def _get(self, url: str, *kargs: Any, **kwargs: Any) -> requests.Response:
        return self._request('GET', url, *kargs, **kwargs)

    def _post(self, url: str, *kargs: Any, **kwargs: Any) -> requests.Response:
        return self._request('POST', url, *kargs, **kwargs)
