from typing import NewType, List, Dict, Any, cast, Optional

from druidapi.client import _common


DruidDatasourceId = NewType('DruidDatasourceId', str)


class CoordinatorClient(_common.CommonClient):
    def rules_get_all(self) -> Dict[DruidDatasourceId, List[Dict[str, Any]]]:
        response = self._get(f'/druid/coordinator/v1/rules/')
        if response.status_code == 200:
            return cast(Dict[DruidDatasourceId, List[Dict[str, Any]]], response.json())

        raise Exception(f'{response.status_code}: {response.text}')

    def rules_get(self, datasource: DruidDatasourceId, full: bool = False) -> List[Dict[str, Any]]:
        params = dict()
        if full:
            params['full'] = ''

        response = self._get(f'/druid/coordinator/v1/rules/{datasource}', params=params)
        if response.status_code == 200:
            return cast(List[Dict[str, Any]], response.json())

        raise Exception(f'{response.status_code}: {response.text}')

    def rules_post(self, datasource: DruidDatasourceId, rules: List[Dict[str, Any]],
                   author: Optional[str] = None, comment: Optional[str] = None) -> None:
        headers = {}
        if author is not None:
            headers['X-Druid-Author'] = str(author)
        if comment is not None:
            headers['X-Druid-Comment'] = str(comment)

        response = self._post(f'/druid/coordinator/v1/rules/{datasource}', json=rules, headers=headers)
        if response.status_code == 200:
            return

        raise Exception(f'{response.status_code}: {response.text}')
